From luca.risolia@studio.unibo.it Sat Jun 25 08:42:33 2005
Date: Sat, 25 Jun 2005 16:30:24 +0200
From: Luca Risolia <luca.risolia@studio.unibo.it>
To: Greg Kroah-Hartman <gregkh@suse.de>
Cc: linux-usb-devel@lists.sourceforge.net
Subject: USB: SN9C10x driver updates
Message-ID: <20050625143024.GA2458@studio.unibo.it>

SN9C10x driver updates.

Changes: + new, - removed, * cleanup, @ bugfix

@ Remove bad get_ctrl()'s
* Documentation updates
+ Add 0x0c45/0x602d to the list of SN9C10x based devices
+ Add support for OV7630 image sensors

Signed-off-by: Luca Risolia <luca.risolia@studio.unibo.it>
Signed-off-by: Greg Kroah-Hartman <gregkh@suse.de>


---
 Documentation/usb/sn9c102.txt          |    4 
 drivers/usb/media/Makefile             |    2 
 drivers/usb/media/sn9c102.h            |    2 
 drivers/usb/media/sn9c102_core.c       |    2 
 drivers/usb/media/sn9c102_ov7630.c     |  394 +++++++++++++++++++++++++++++++++
 drivers/usb/media/sn9c102_sensor.h     |   16 +
 drivers/usb/media/sn9c102_tas5110c1b.c |   21 -
 drivers/usb/media/sn9c102_tas5130d1b.c |   27 --
 8 files changed, 417 insertions(+), 51 deletions(-)

--- gregkh-2.6.orig/Documentation/usb/sn9c102.txt	2005-06-17 12:48:29.000000000 -0700
+++ gregkh-2.6/Documentation/usb/sn9c102.txt	2005-07-01 14:17:21.000000000 -0700
@@ -297,6 +297,7 @@
 0x0c45     0x602a
 0x0c45     0x602b
 0x0c45     0x602c
+0x0c45     0x602d
 0x0c45     0x6030
 0x0c45     0x6080
 0x0c45     0x6082
@@ -333,6 +334,7 @@
 -----       ------------
 HV7131D     Hynix Semiconductor, Inc.
 MI-0343     Micron Technology, Inc.
+OV7630      OmniVision Technologies, Inc.
 PAS106B     PixArt Imaging, Inc.
 PAS202BCB   PixArt Imaging, Inc.
 TAS5110C1B  Taiwan Advanced Sensor Corporation
@@ -470,9 +472,11 @@
 - Luca Capello for the donation of a webcam;
 - Joao Rodrigo Fuzaro, Joao Limirio, Claudio Filho and Caio Begotti for the
   donation of a webcam;
+- Jon Hollstrom for the donation of a webcam;
 - Carlos Eduardo Medaglia Dyonisio, who added the support for the PAS202BCB
   image sensor;
 - Stefano Mozzi, who donated 45 EU;
+- Andrew Pearce for the donation of a webcam;
 - Bertrik Sikken, who reverse-engineered and documented the Huffman compression
   algorithm used in the SN9C10x controllers and implemented the first decoder;
 - Mizuno Takafumi for the donation of a webcam;
--- gregkh-2.6.orig/drivers/usb/media/Makefile	2005-06-17 12:48:29.000000000 -0700
+++ gregkh-2.6/drivers/usb/media/Makefile	2005-07-01 14:17:21.000000000 -0700
@@ -2,7 +2,7 @@
 # Makefile for USB Media drivers
 #
 
-sn9c102-objs	:= sn9c102_core.o sn9c102_hv7131d.o sn9c102_mi0343.o sn9c102_pas106b.o sn9c102_pas202bcb.o sn9c102_tas5110c1b.o sn9c102_tas5130d1b.o
+sn9c102-objs	:= sn9c102_core.o sn9c102_hv7131d.o sn9c102_mi0343.o sn9c102_ov7630.o sn9c102_pas106b.o sn9c102_pas202bcb.o sn9c102_tas5110c1b.o sn9c102_tas5130d1b.o
 
 obj-$(CONFIG_USB_DABUSB)	+= dabusb.o
 obj-$(CONFIG_USB_DSBR)		+= dsbr100.o
--- gregkh-2.6.orig/drivers/usb/media/sn9c102_core.c	2005-06-17 12:48:29.000000000 -0700
+++ gregkh-2.6/drivers/usb/media/sn9c102_core.c	2005-07-01 14:17:21.000000000 -0700
@@ -429,7 +429,7 @@
 }
 
 
-static int 
+int
 sn9c102_i2c_try_write(struct sn9c102_device* cam,
                       struct sn9c102_sensor* sensor, u8 address, u8 value)
 {
--- gregkh-2.6.orig/drivers/usb/media/sn9c102.h	2005-06-17 12:48:29.000000000 -0700
+++ gregkh-2.6/drivers/usb/media/sn9c102.h	2005-07-01 14:17:21.000000000 -0700
@@ -56,7 +56,7 @@
 #define SN9C102_MODULE_AUTHOR   "(C) 2004-2005 Luca Risolia"
 #define SN9C102_AUTHOR_EMAIL    "<luca.risolia@studio.unibo.it>"
 #define SN9C102_MODULE_LICENSE  "GPL"
-#define SN9C102_MODULE_VERSION  "1:1.24"
+#define SN9C102_MODULE_VERSION  "1:1.24a"
 #define SN9C102_MODULE_VERSION_CODE  KERNEL_VERSION(1, 0, 24)
 
 enum sn9c102_bridge {
--- /dev/null	1970-01-01 00:00:00.000000000 +0000
+++ gregkh-2.6/drivers/usb/media/sn9c102_ov7630.c	2005-07-01 14:17:21.000000000 -0700
@@ -0,0 +1,394 @@
+/***************************************************************************
+ * Plug-in for OV7630 image sensor connected to the SN9C10x PC Camera      *
+ * Controllers                                                             *
+ *                                                                         *
+ * Copyright (C) 2005 by Luca Risolia <luca.risolia@studio.unibo.it>       *
+ *                                                                         *
+ * This program is free software; you can redistribute it and/or modify    *
+ * it under the terms of the GNU General Public License as published by    *
+ * the Free Software Foundation; either version 2 of the License, or       *
+ * (at your option) any later version.                                     *
+ *                                                                         *
+ * This program is distributed in the hope that it will be useful,         *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
+ * GNU General Public License for more details.                            *
+ *                                                                         *
+ * You should have received a copy of the GNU General Public License       *
+ * along with this program; if not, write to the Free Software             *
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.               *
+ ***************************************************************************/
+
+#include "sn9c102_sensor.h"
+
+
+static struct sn9c102_sensor ov7630;
+
+
+static int ov7630_init(struct sn9c102_device* cam)
+{
+	int err = 0;
+
+	err += sn9c102_write_reg(cam, 0x00, 0x14);
+	err += sn9c102_write_reg(cam, 0x60, 0x17);
+	err += sn9c102_write_reg(cam, 0x0f, 0x18);
+	err += sn9c102_write_reg(cam, 0x50, 0x19);
+
+	err += sn9c102_i2c_write(cam, 0x12, 0x8d);
+	err += sn9c102_i2c_write(cam, 0x11, 0x00);
+	err += sn9c102_i2c_write(cam, 0x15, 0x34);
+	err += sn9c102_i2c_write(cam, 0x16, 0x03);
+	err += sn9c102_i2c_write(cam, 0x17, 0x1c);
+	err += sn9c102_i2c_write(cam, 0x18, 0xbd);
+	err += sn9c102_i2c_write(cam, 0x19, 0x06);
+	err += sn9c102_i2c_write(cam, 0x1a, 0xf6);
+	err += sn9c102_i2c_write(cam, 0x1b, 0x04);
+	err += sn9c102_i2c_write(cam, 0x20, 0x44);
+	err += sn9c102_i2c_write(cam, 0x23, 0xee);
+	err += sn9c102_i2c_write(cam, 0x26, 0xa0);
+	err += sn9c102_i2c_write(cam, 0x27, 0x9a);
+	err += sn9c102_i2c_write(cam, 0x28, 0x20);
+	err += sn9c102_i2c_write(cam, 0x29, 0x30);
+	err += sn9c102_i2c_write(cam, 0x2f, 0x3d);
+	err += sn9c102_i2c_write(cam, 0x30, 0x24);
+	err += sn9c102_i2c_write(cam, 0x32, 0x86);
+	err += sn9c102_i2c_write(cam, 0x60, 0xa9);
+	err += sn9c102_i2c_write(cam, 0x61, 0x42);
+	err += sn9c102_i2c_write(cam, 0x65, 0x00);
+	err += sn9c102_i2c_write(cam, 0x69, 0x38);
+	err += sn9c102_i2c_write(cam, 0x6f, 0x88);
+	err += sn9c102_i2c_write(cam, 0x70, 0x0b);
+	err += sn9c102_i2c_write(cam, 0x71, 0x00);
+	err += sn9c102_i2c_write(cam, 0x74, 0x21);
+	err += sn9c102_i2c_write(cam, 0x7d, 0xf7);
+
+	return err;
+}
+
+
+static int ov7630_set_ctrl(struct sn9c102_device* cam,
+                           const struct v4l2_control* ctrl)
+{
+	int err = 0;
+
+	switch (ctrl->id) {
+	case V4L2_CID_EXPOSURE:
+		err += sn9c102_i2c_write(cam, 0x10, ctrl->value >> 2);
+		err += sn9c102_i2c_write(cam, 0x76, ctrl->value & 0x03);
+		break;
+	case V4L2_CID_RED_BALANCE:
+		err += sn9c102_i2c_write(cam, 0x02, ctrl->value);
+		break;
+	case V4L2_CID_BLUE_BALANCE:
+		err += sn9c102_i2c_write(cam, 0x03, ctrl->value);
+		break;
+	case V4L2_CID_GAIN:
+		err += sn9c102_i2c_write(cam, 0x00, ctrl->value);
+		break;
+	case V4L2_CID_CONTRAST:
+		err += ctrl->value ? sn9c102_i2c_write(cam, 0x05,
+		                                       (ctrl->value-1) | 0x20)
+		                   : sn9c102_i2c_write(cam, 0x05, 0x00);
+		break;
+	case V4L2_CID_BRIGHTNESS:
+		err += sn9c102_i2c_write(cam, 0x06, ctrl->value);
+		break;
+	case V4L2_CID_SATURATION:
+		err += sn9c102_i2c_write(cam, 0x03, ctrl->value << 4);
+		break;
+	case V4L2_CID_HUE:
+		err += ctrl->value ? sn9c102_i2c_write(cam, 0x04,
+		                                       (ctrl->value-1) | 0x20)
+		                   : sn9c102_i2c_write(cam, 0x04, 0x00);
+		break;
+	case V4L2_CID_DO_WHITE_BALANCE:
+		err += sn9c102_i2c_write(cam, 0x0c, ctrl->value);
+		break;
+	case V4L2_CID_WHITENESS:
+		err += sn9c102_i2c_write(cam, 0x0d, ctrl->value);
+		break;
+	case V4L2_CID_AUTO_WHITE_BALANCE:
+		err += sn9c102_i2c_write(cam, 0x12, (ctrl->value << 2) | 0x09);
+		break;
+	case V4L2_CID_AUTOGAIN:
+		err += sn9c102_i2c_write(cam, 0x13, ctrl->value);
+		break;
+	case V4L2_CID_VFLIP:
+		err += sn9c102_i2c_write(cam, 0x75, 0x0e | (ctrl->value << 7));
+		break;
+	case V4L2_CID_BLACK_LEVEL:
+		err += sn9c102_i2c_write(cam, 0x25, ctrl->value);
+		break;
+	case SN9C102_V4L2_CID_BRIGHT_LEVEL:
+		err += sn9c102_i2c_write(cam, 0x24, ctrl->value);
+		break;
+	case SN9C102_V4L2_CID_GAMMA:
+		err += sn9c102_i2c_write(cam, 0x14, (ctrl->value << 2) | 0x80);
+		break;
+	case SN9C102_V4L2_CID_BAND_FILTER:
+		err += sn9c102_i2c_write(cam, 0x2d, ctrl->value << 2);
+		break;
+	default:
+		return -EINVAL;
+	}
+
+	return err ? -EIO : 0;
+}
+
+
+static int ov7630_set_crop(struct sn9c102_device* cam,
+                           const struct v4l2_rect* rect)
+{
+	struct sn9c102_sensor* s = &ov7630;
+	int err = 0;
+	u8 v_start = (u8)(rect->top - s->cropcap.bounds.top) + 1;
+
+	err += sn9c102_write_reg(cam, v_start, 0x13);
+
+	return err;
+}
+
+
+static int ov7630_set_pix_format(struct sn9c102_device* cam,
+                                 const struct v4l2_pix_format* pix)
+{
+	int err = 0;
+
+	if (pix->pixelformat == V4L2_PIX_FMT_SN9C10X)
+		err += sn9c102_write_reg(cam, 0x20, 0x19);
+	else
+		err += sn9c102_write_reg(cam, 0x50, 0x19);
+
+	return err;
+}
+
+
+static struct sn9c102_sensor ov7630 = {
+	.name = "OV7630",
+	.maintainer = "Luca Risolia <luca.risolia@studio.unibo.it>",
+	.sysfs_ops = SN9C102_I2C_WRITE,
+	.frequency = SN9C102_I2C_100KHZ,
+	.interface = SN9C102_I2C_2WIRES,
+	.i2c_slave_id = 0x21,
+	.init = &ov7630_init,
+	.qctrl = {
+		{
+			.id = V4L2_CID_GAIN,
+			.type = V4L2_CTRL_TYPE_INTEGER,
+			.name = "global gain",
+			.minimum = 0x00,
+			.maximum = 0x3f,
+			.step = 0x01,
+			.default_value = 0x14,
+			.flags = 0,
+		},
+		{
+			.id = V4L2_CID_HUE,
+			.type = V4L2_CTRL_TYPE_INTEGER,
+			.name = "hue",
+			.minimum = 0x00,
+			.maximum = 0x1f+1,
+			.step = 0x01,
+			.default_value = 0x00,
+			.flags = 0,
+		},
+		{
+			.id = V4L2_CID_SATURATION,
+			.type = V4L2_CTRL_TYPE_INTEGER,
+			.name = "saturation",
+			.minimum = 0x00,
+			.maximum = 0x0f,
+			.step = 0x01,
+			.default_value = 0x08,
+			.flags = 0,
+		},
+		{
+			.id = V4L2_CID_CONTRAST,
+			.type = V4L2_CTRL_TYPE_INTEGER,
+			.name = "contrast",
+			.minimum = 0x00,
+			.maximum = 0x1f+1,
+			.step = 0x01,
+			.default_value = 0x00,
+			.flags = 0,
+		},
+		{
+			.id = V4L2_CID_EXPOSURE,
+			.type = V4L2_CTRL_TYPE_INTEGER,
+			.name = "exposure",
+			.minimum = 0x000,
+			.maximum = 0x3ff,
+			.step = 0x001,
+			.default_value = 0x83<<2,
+			.flags = 0,
+		},
+		{
+			.id = V4L2_CID_RED_BALANCE,
+			.type = V4L2_CTRL_TYPE_INTEGER,
+			.name = "red balance",
+			.minimum = 0x00,
+			.maximum = 0xff,
+			.step = 0x01,
+			.default_value = 0x3a,
+			.flags = 0,
+		},
+		{
+			.id = V4L2_CID_BLUE_BALANCE,
+			.type = V4L2_CTRL_TYPE_INTEGER,
+			.name = "blue balance",
+			.minimum = 0x00,
+			.maximum = 0xff,
+			.step = 0x01,
+			.default_value = 0x77,
+			.flags = 0,
+		},
+		{
+			.id = V4L2_CID_BRIGHTNESS,
+			.type = V4L2_CTRL_TYPE_INTEGER,
+			.name = "brightness",
+			.minimum = 0x00,
+			.maximum = 0xff,
+			.step = 0x01,
+			.default_value = 0xa0,
+			.flags = 0,
+		},
+		{
+			.id = V4L2_CID_DO_WHITE_BALANCE,
+			.type = V4L2_CTRL_TYPE_INTEGER,
+			.name = "white balance background: blue",
+			.minimum = 0x00,
+			.maximum = 0x3f,
+			.step = 0x01,
+			.default_value = 0x20,
+			.flags = 0,
+		},
+		{
+			.id = V4L2_CID_WHITENESS,
+			.type = V4L2_CTRL_TYPE_INTEGER,
+			.name = "white balance background: red",
+			.minimum = 0x00,
+			.maximum = 0x3f,
+			.step = 0x01,
+			.default_value = 0x20,
+			.flags = 0,
+		},
+		{
+			.id = V4L2_CID_AUTO_WHITE_BALANCE,
+			.type = V4L2_CTRL_TYPE_BOOLEAN,
+			.name = "auto white balance",
+			.minimum = 0x00,
+			.maximum = 0x01,
+			.step = 0x01,
+			.default_value = 0x01,
+			.flags = 0,
+		},
+		{
+			.id = V4L2_CID_AUTOGAIN,
+			.type = V4L2_CTRL_TYPE_INTEGER,
+			.name = "gain & exposure mode",
+			.minimum = 0x00,
+			.maximum = 0x03,
+			.step = 0x01,
+			.default_value = 0x00,
+			.flags = 0,
+		},
+		{
+			.id = V4L2_CID_VFLIP,
+			.type = V4L2_CTRL_TYPE_BOOLEAN,
+			.name = "vertical flip",
+			.minimum = 0x00,
+			.maximum = 0x01,
+			.step = 0x01,
+			.default_value = 0x01,
+			.flags = 0,
+		},
+		{
+			.id = V4L2_CID_BLACK_LEVEL,
+			.type = V4L2_CTRL_TYPE_INTEGER,
+			.name = "black pixel ratio",
+			.minimum = 0x01,
+			.maximum = 0x9a,
+			.step = 0x01,
+			.default_value = 0x8a,
+			.flags = 0,
+		},
+		{
+			.id = SN9C102_V4L2_CID_BRIGHT_LEVEL,
+			.type = V4L2_CTRL_TYPE_INTEGER,
+			.name = "bright pixel ratio",
+			.minimum = 0x01,
+			.maximum = 0x9a,
+			.step = 0x01,
+			.default_value = 0x10,
+			.flags = 0,
+		},
+		{
+			.id = SN9C102_V4L2_CID_BAND_FILTER,
+			.type = V4L2_CTRL_TYPE_BOOLEAN,
+			.name = "band filter",
+			.minimum = 0x00,
+			.maximum = 0x01,
+			.step = 0x01,
+			.default_value = 0x00,
+			.flags = 0,
+		},
+		{
+			.id = SN9C102_V4L2_CID_GAMMA,
+			.type = V4L2_CTRL_TYPE_BOOLEAN,
+			.name = "rgb gamma",
+			.minimum = 0x00,
+			.maximum = 0x01,
+			.step = 0x01,
+			.default_value = 0x00,
+			.flags = 0,
+		},
+	},
+	.set_ctrl = &ov7630_set_ctrl,
+	.cropcap = {
+		.bounds = {
+			.left = 0,
+			.top = 0,
+			.width = 640,
+			.height = 480,
+		},
+		.defrect = {
+			.left = 0,
+			.top = 0,
+			.width = 640,
+			.height = 480,
+		},
+	},
+	.set_crop = &ov7630_set_crop,
+	.pix_format = {
+		.width = 640,
+		.height = 480,
+		.pixelformat = V4L2_PIX_FMT_SBGGR8,
+		.priv = 8,
+	},
+	.set_pix_format = &ov7630_set_pix_format
+};
+
+
+int sn9c102_probe_ov7630(struct sn9c102_device* cam)
+{
+	int err = 0;
+
+	sn9c102_attach_sensor(cam, &ov7630);
+
+	if (le16_to_cpu(ov7630.usbdev->descriptor.idProduct) != 0x608f &&
+	    le16_to_cpu(ov7630.usbdev->descriptor.idProduct) != 0x602c)
+		return -ENODEV;
+
+	err += sn9c102_write_reg(cam, 0x01, 0x01);
+	err += sn9c102_write_reg(cam, 0x00, 0x01);
+	err += sn9c102_write_reg(cam, 0x28, 0x17);
+
+	if (err)
+		return -EIO;
+
+	err += sn9c102_i2c_write(cam, 0x0b, 0);
+	if (err)
+		return -ENODEV;
+
+	return 0;
+}
--- gregkh-2.6.orig/drivers/usb/media/sn9c102_sensor.h	2005-06-17 12:48:29.000000000 -0700
+++ gregkh-2.6/drivers/usb/media/sn9c102_sensor.h	2005-07-01 14:17:21.000000000 -0700
@@ -64,6 +64,7 @@
 */
 extern int sn9c102_probe_hv7131d(struct sn9c102_device* cam);
 extern int sn9c102_probe_mi0343(struct sn9c102_device* cam);
+extern int sn9c102_probe_ov7630(struct sn9c102_device* cam);
 extern int sn9c102_probe_pas106b(struct sn9c102_device* cam);
 extern int sn9c102_probe_pas202bcb(struct sn9c102_device* cam);
 extern int sn9c102_probe_tas5110c1b(struct sn9c102_device* cam);
@@ -80,6 +81,7 @@
 	&sn9c102_probe_pas106b, /* strong detection based on SENSOR ids */    \
 	&sn9c102_probe_pas202bcb, /* strong detection based on SENSOR ids */  \
 	&sn9c102_probe_hv7131d, /* strong detection based on SENSOR ids */    \
+	&sn9c102_probe_ov7630, /* detection mostly based on USB pid/vid */    \
 	&sn9c102_probe_tas5110c1b, /* detection based on USB pid/vid */       \
 	&sn9c102_probe_tas5130d1b, /* detection based on USB pid/vid */       \
 	NULL,                                                                 \
@@ -103,7 +105,8 @@
 	{ USB_DEVICE(0x0c45, 0x6029), }, /* PAS106B */                        \
 	{ USB_DEVICE(0x0c45, 0x602a), }, /* HV7131D */                        \
 	{ USB_DEVICE(0x0c45, 0x602b), }, /* MI-0343 */                        \
-	{ USB_DEVICE(0x0c45, 0x602c), }, /* OV7620 */                         \
+	{ USB_DEVICE(0x0c45, 0x602c), }, /* OV7630 */                         \
+	{ USB_DEVICE(0x0c45, 0x602d), },                                      \
 	{ USB_DEVICE(0x0c45, 0x6030), }, /* MI03x */                          \
 	{ USB_DEVICE(0x0c45, 0x6080), },                                      \
 	{ USB_DEVICE(0x0c45, 0x6082), }, /* MI0343 and MI0360 */              \
@@ -145,6 +148,8 @@
 */
 
 /* The "try" I2C I/O versions are used when probing the sensor */
+extern int sn9c102_i2c_try_write(struct sn9c102_device*,struct sn9c102_sensor*,
+                                 u8 address, u8 value);
 extern int sn9c102_i2c_try_read(struct sn9c102_device*,struct sn9c102_sensor*,
                                 u8 address);
 
@@ -201,6 +206,8 @@
 	SN9C102_I2C_3WIRES,
 };
 
+#define SN9C102_MAX_CTRLS V4L2_CID_LASTP1-V4L2_CID_BASE+10
+
 struct sn9c102_sensor {
 	char name[32], /* sensor name */
 	     maintainer[64]; /* name of the mantainer <email> */
@@ -243,7 +250,7 @@
 	   sensor according to the default configuration structures below.
 	*/
 
-	struct v4l2_queryctrl qctrl[V4L2_CID_LASTP1-V4L2_CID_BASE];
+	struct v4l2_queryctrl qctrl[SN9C102_MAX_CTRLS];
 	/*
 	   Optional list of default controls, defined as indicated in the 
 	   V4L2 API. Menu type controls are not handled by this interface.
@@ -356,7 +363,7 @@
 	   core module to store successfully updated values of the above
 	   settings, for rollbacks..etc..in case of errors during atomic I/O
 	*/
-	struct v4l2_queryctrl _qctrl[V4L2_CID_LASTP1-V4L2_CID_BASE];
+	struct v4l2_queryctrl _qctrl[SN9C102_MAX_CTRLS];
 	struct v4l2_rect _rect;
 };
 
@@ -367,5 +374,8 @@
 #define SN9C102_V4L2_CID_GREEN_BALANCE V4L2_CID_PRIVATE_BASE + 1
 #define SN9C102_V4L2_CID_RESET_LEVEL V4L2_CID_PRIVATE_BASE + 2
 #define SN9C102_V4L2_CID_PIXEL_BIAS_VOLTAGE V4L2_CID_PRIVATE_BASE + 3
+#define SN9C102_V4L2_CID_GAMMA V4L2_CID_PRIVATE_BASE + 4
+#define SN9C102_V4L2_CID_BAND_FILTER V4L2_CID_PRIVATE_BASE + 5
+#define SN9C102_V4L2_CID_BRIGHT_LEVEL V4L2_CID_PRIVATE_BASE + 6
 
 #endif /* _SN9C102_SENSOR_H_ */
--- gregkh-2.6.orig/drivers/usb/media/sn9c102_tas5110c1b.c	2005-06-17 12:48:29.000000000 -0700
+++ gregkh-2.6/drivers/usb/media/sn9c102_tas5110c1b.c	2005-07-01 14:17:21.000000000 -0700
@@ -24,8 +24,6 @@
 
 static struct sn9c102_sensor tas5110c1b;
 
-static struct v4l2_control tas5110c1b_gain;
-
 
 static int tas5110c1b_init(struct sn9c102_device* cam)
 {
@@ -46,21 +44,6 @@
 }
 
 
-static int tas5110c1b_get_ctrl(struct sn9c102_device* cam, 
-                               struct v4l2_control* ctrl)
-{
-	switch (ctrl->id) {
-	case V4L2_CID_GAIN:
-		ctrl->value = tas5110c1b_gain.value;
-		break;
-	default:
-		return -EINVAL;
-	}
-
-	return 0;
-}
-
-
 static int tas5110c1b_set_ctrl(struct sn9c102_device* cam, 
                                const struct v4l2_control* ctrl)
 {
@@ -68,8 +51,7 @@
 
 	switch (ctrl->id) {
 	case V4L2_CID_GAIN:
-		if (!(err += sn9c102_i2c_write(cam, 0x20, 0xf6 - ctrl->value)))
-			tas5110c1b_gain.value = ctrl->value;
+		err += sn9c102_i2c_write(cam, 0x20, 0xf6 - ctrl->value);
 		break;
 	default:
 		return -EINVAL;
@@ -147,7 +129,6 @@
 			.height = 288,
 		},
 	},
-	.get_ctrl = &tas5110c1b_get_ctrl,
 	.set_crop = &tas5110c1b_set_crop,
 	.pix_format = {
 		.width = 352,
--- gregkh-2.6.orig/drivers/usb/media/sn9c102_tas5130d1b.c	2005-06-17 12:48:29.000000000 -0700
+++ gregkh-2.6/drivers/usb/media/sn9c102_tas5130d1b.c	2005-07-01 14:17:21.000000000 -0700
@@ -24,8 +24,6 @@
 
 static struct sn9c102_sensor tas5130d1b;
 
-static struct v4l2_control tas5130d1b_gain, tas5130d1b_exposure;
-
 
 static int tas5130d1b_init(struct sn9c102_device* cam)
 {
@@ -44,24 +42,6 @@
 }
 
 
-static int tas5130d1b_get_ctrl(struct sn9c102_device* cam, 
-                               struct v4l2_control* ctrl)
-{
-	switch (ctrl->id) {
-	case V4L2_CID_GAIN:
-		ctrl->value = tas5130d1b_gain.value;
-		break;
-	case V4L2_CID_EXPOSURE:
-		ctrl->value = tas5130d1b_exposure.value;
-		break;
-	default:
-		return -EINVAL;
-	}
-
-	return 0;
-}
-
-
 static int tas5130d1b_set_ctrl(struct sn9c102_device* cam, 
                                const struct v4l2_control* ctrl)
 {
@@ -69,12 +49,10 @@
 
 	switch (ctrl->id) {
 	case V4L2_CID_GAIN:
-		if (!(err += sn9c102_i2c_write(cam, 0x20, 0xf6 - ctrl->value)))
-			tas5130d1b_gain.value = ctrl->value;
+		err += sn9c102_i2c_write(cam, 0x20, 0xf6 - ctrl->value);
 		break;
 	case V4L2_CID_EXPOSURE:
-		if (!(err += sn9c102_i2c_write(cam, 0x40, 0x47 - ctrl->value)))
-			tas5130d1b_exposure.value = ctrl->value;
+		err += sn9c102_i2c_write(cam, 0x40, 0x47 - ctrl->value);
 		break;
 	default:
 		return -EINVAL;
@@ -147,7 +125,6 @@
 			.flags = 0,
 		},
 	},
-	.get_ctrl = &tas5130d1b_get_ctrl,
 	.set_ctrl = &tas5130d1b_set_ctrl,
 	.cropcap = {
 		.bounds = {
